@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(Ryu::Node);
 
-our $VERSION = '2.002'; # VERSION
+our $VERSION = '2.003'; # VERSION
 our $AUTHORITY = 'cpan:TEAM'; # AUTHORITY
 
 =head1 NAME
@@ -1158,12 +1158,18 @@ sub ordered_futures {
         $src->completed->done unless %pending or $src_completed->is_ready;
     });
 
+    $src_completed->on_ready(sub {
+        my @pending = values %pending;
+        %pending = ();
+        $_->cancel for grep { $_ and not $_->is_ready } @pending;
+    });
     $self->each(sub {
-        my $k = Scalar::Util::refaddr $_;
-        $pending{$k} = 1;
-        $log->tracef('Ordered futures has %d pending', 0 + keys %pending);
         my $f = $_;
-        $src_completed->on_ready(sub { $f->cancel });
+        my $k = Scalar::Util::refaddr $f;
+        # This will keep a copy of the Future around until the
+        # ->is_ready callback removes it
+        $pending{$k} = $f;
+        $log->tracef('Ordered futures has %d pending', 0 + keys %pending);
         $_->on_done(sub {
             my @pending = @_;
             while(@pending and not $src_completed->is_ready) {
@@ -1177,7 +1183,6 @@ sub ordered_futures {
               return if %pending;
               $src_completed->done if $all_finished and not $src_completed->is_ready;
           })
-          ->retain
     });
     return $src;
 }
